@@ -3,8 +3,8 @@
 ;; Copyright (C) 2025 Pablo González Carrizo
 
 ;; Author: Pablo González Carrizo <unmonoqueteclea@gmail.com>
-;; Package-Version: 20251025.939
-;; Package-Revision: 617329b979fd
+;; Package-Version: 20251114.1916
+;; Package-Revision: 227c59839f90
 ;; Created: 2025-02-16
 ;; URL: https://github.com/unmonoqueteclea/jira.el
 ;; Package-Requires: ((emacs "29.1") (request "0.3.0") (tablist "1.0") (transient "0.8.3") (magit-section "4.2.0"))
@@ -33,9 +33,10 @@
 ;;; Code:
 
 (require 'jira-issues)
+(require 'jira-export)
 (require 'jira-tempo)
 
-(defconst jira-version "jira.el v2.10.3" "jira.el package version.")
+(defconst jira-version "jira.el v2.14.0" "Jira.el package version.")
 
 (provide 'jira)
 
